#!/usr/bin/bash
# Atomicorp, Inc
# Copyrigth 2024
# Summary: Common shell functions

# download_rules <URL> <PATH TO SAVE> 
download() {
    if [ $# -lt 2 ]; then
        echo "Usage: download [-k|--insecure] <URL> <destination_path>"
        return 1
    fi

    local insecure=false
    if [ "$1" = "-k" ] || [ "$1" = "--insecure" ]; then
        insecure=true
        shift
    fi

    url=$1
    destination=$2

    if command -v curl >/dev/null; then
        if [ "$insecure" = true ]; then
            curl -o "$destination" -k "$url" || { echo "Error downloading file from $url"; return 1; }
        else
            curl -o "$destination" "$url" || { echo "Error downloading file from $url"; return 1; }
        fi
    elif command -v wget >/dev/null; then
        wget -O "$destination" "$url" || { echo "Error downloading file from $url"; return 1; }
    else
        echo "Error: Neither curl nor wget found. Please install either curl or wget."
        return 1
    fi
}



get_hub_ip() {
    HUB_IP=$(grep -o -P '(?<=<address>).*(?=</address>)' /var/ossec/etc/ossec.conf)
}


# get_freshclam_conf <configfile>
get_freshclam_conf() {
    for line in $(cat $CONFIG_FILE ); do
        if [[ ${line} == setting,* ]]; then
            CLAM_SETTING=$(echo $line |awk -F, '{ print $2}')
            CLAM_TOKEN=$(echo $line |awk -F, '{ print $3}')
            if [[ ${CLAM_SETTING} == freshclam_* ]]; then
                if [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "atomic" ]]; then
                    FRESHCLAM_FEED_TYPE="atomic"
                elif [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "official" ]]; then
                    FRESHCLAM_FEED_TYPE="official"
                elif [[ ${CLAM_SETTING} == "freshclam_feed_type" ]] && [[ ${CLAM_TOKEN} == "hub" ]]; then
                    FRESHCLAM_FEED_TYPE="hub"
                fi

                if [[ ${CLAM_SETTING} == "freshclam_linux_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
                    FRESHCLAM_LINUX_SIGS="yes"
                fi
                if [[ ${CLAM_SETTING} == "freshclam_windows_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
                    FRESHCLAM_WINDOWS_SIGS="yes"
                fi
                if [[ ${CLAM_SETTING} == "freshclam_android_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
                    FRESHCLAM_ANDROID_SIGS="yes"
                fi
                if [[ ${CLAM_SETTING} == "freshclam_osx_sigs" ]] && [[ ${CLAM_TOKEN} == "true" ]]; then
                    FRESHCLAM_OSX_SIGS="yes"
                fi
            fi
        fi
    done

    # Identify the feed owner
    if id clamupdate &>/dev/null; then
        CLAMAV_OWNER=clamupdate
        CLAMAV_GROUP=clamupdate
    elif id clamav &>/dev/null; then
        CLAMAV_OWNER=clamav
        CLAMAV_GROUP=root
    else
        CLAMAV_OWNER=clamav
        CLAMAV_GROUP=root
    fi

}
