#!/bin/bash
# Copyright Atomicorp, 2021
# All Rights Reserved

# Globals
VERSION="0.2"
FW_POLICY=/var/ossec/etc/shared/firewall-policy
if [ ! -f /var/ossec/etc/shared/firewall-policy ]; then
	exit 0
fi


# Detect OS
function dist_detection() {

	PKG=rpm
        if [ -f /etc/redhat-release ]; then
                RELEASE=/etc/redhat-release
        elif [ -f /etc/os-release ]; then
                RELEASE=/etc/os-release
        elif [ -f /etc/openvz-release ]; then
                RELEASE=/etc/openvz-release
        elif [ -f /etc/SuSE-release ]; then
                RELEASE=/etc/SuSE-release
        elif [ -f /etc/os-release ]; then
                RELEASE=/etc/os-release
        elif [ -f /etc/lsb-release ]; then
                RELEASE=/etc/lsb-release
        elif [ -f /etc/debian_version ]; then
                RELEASE=/etc/debian_version
        elif [ -f /etc/openvz-release ]; then
                RELEASE=/etc/openvz-release
        elif [ -f /etc/virtuozzo-release ]; then
                RELEASE=/etc/virtuozzo-release
        fi

        if [[ $OSTYPE == "aix"* ]]; then
                PKG=aix
	elif egrep -q "(release 5)" $RELEASE ; then
                DIST="el5"
        elif egrep -q "(release 6|release 2012)" $RELEASE ; then
                DIST="el6"
        elif egrep -q "(release 7|release 2014)" $RELEASE ; then
                DIST="el7"
        elif egrep -q "(release 8)" $RELEASE ; then
                DIST="el8"
        elif egrep -q "Red Hat Enterprise Linux.* 7" $RELEASE ; then
                DIST="el7"
        elif egrep -q "Red Hat Enterprise Linux.* 8" $RELEASE ; then
                DIST="el8"
        elif egrep -q "(Amazon Linux 2)" $RELEASE; then
                DIST="amazon"
        elif egrep -q "(Amazon Linux AMI|Amazon)" $RELEASE ; then
                DIST="el6"
        elif egrep -q "wheezy" $RELEASE ; then
                DIST="debian"
                PKG=deb
        elif egrep -q "jessie" $RELEASE ; then
                DIST="debian"
                PKG=deb
        elif egrep -q "stretch" $RELEASE ; then
                DIST="debian"
                PKG=deb
        elif egrep -q "lucid" $RELEASE ; then
                DIST="debian"
                PKG=deb
        elif egrep -q "precise" $RELEASE ; then
                DIST="debian"
                PKG=deb
        elif egrep -q "Raring Ringtail" $RELEASE ; then
                DIST="debian"
                PKG=deb
        elif egrep -q "Trusty Tahr" $RELEASE ; then
                DIST="ubuntu"
                PKG=deb
        elif egrep -q "Xenial" $RELEASE ; then
                DIST="ubuntu"
                PKG=deb
	else
		echo "ERROR A1000: Could not identify Operating System"
		exit 1
        fi

}



#AIX
function aix_firewall() {

	# Dependencies
	#   bos.net.ipsec.keymgt
	#   bos.net.ipsec.rte
	if ! lslpp -l |egrep -q "bos.net.ipsec.keymgt|bos.net.ipsec.rte" ; then
		echo "ERROR A2011: Firewall (AIX) support is not available"
		exit 1
	fi
	RMFILT="/usr/sbin/rmfilt"
	GENFILT="/usr/sbin/genfilt"
	MKFILT="/usr/sbin/mkfilt"
	LSFILT="/usr/sbin/lsfilt"

	## Clear existing policy
	$RMFILT -v 4 -n all
	$RMFILT -v 6 -n all

	# Initialize firewall
	$MKFILT -c ipsec -t 4 >/dev/null 2>&1
	$MKFILT -v 4 -u -z P
	$MKFILT -v 6 -u -z P

	# Create policy
	# -a P permit
	# -a D deny
	# -a H SHUN_HOST filter rule
	# -a S SHUN_PORT filter rule
	# -c tcp/udp/etc
	# -D description
	# -d dstip
	# -e expiration_time, used with SHUN_HOST/PORT
	# -f fragmentation control [default: Y]
	# -g source routing. Only applies to permit rules
	# -i <interface>
	# -l logging, Y or N [default: N]
	# -M dest subnet mask / -m src subnetmask
	# -o source port modifier (eq, lt, gt, etc). 
	# -O dst port modifier, (eq,lt,gt,etc) required
	# -p source port
	# -P dest port
	# -r routing? (R) forward, (L) local, (B) both [default: B]
	# -s srcip
	# -t id of the tunnel?
	# -v 4/6
	# -w direction, (I) incoming, (O) outgoing or (B) [default: 
	# -x / -X

	#permit,4,0.0.0.0,0.0.0.0,0.0.0.0,0.0.0.0,tcp,eq,22,inbound,"example rule",no
	#deny,4,0.0.0.0,0.0.0.0,0.0.0.0,0.0.0.0,tcp,eq,23,inbound,"example block rule",log


	while read line ; do
		IPV=$(echo $line | awk -F, '{print $2}')
		SRCIP=$(echo $line | awk -F, '{print $3}')
		SRCIP_MASK=$(echo $line | awk -F, '{print $4}')
		DSTIP=$(echo $line | awk -F, '{print $5}')
		DSTIP_MASK=$(echo $line | awk -F, '{print $6}')
		PROTOCOL=$(echo $line | awk -F, '{print $7}')
		CONDITION=$(echo $line | awk -F, '{print $8}')
		PORT=$(echo $line | awk -F, '{print $9}')
		DIRECTION=$(echo $line | awk -F, '{print $10}')
		if [[ $DIRECTION == inbound ]]; then
			DIRECTION=I 
		elif [[ $DIRECTION == outbound ]]; then
			DIRECTION=O
		else
			DIRECTION=B
		fi
		COMMENT=$(echo $line | awk -F, '{print $11}')
		LOG=$(echo $line | awk -F, '{print $12}')
		if [[ $LOG == log ]]; then
			LOG=Y
		else
			LOG=N
		fi

		if  [[ ${line} == disabled* ]]; then
			exit
		elif  [[ ${line} == permit* ]]; then
			genfilt -v $IPV -a P -s $SRCIP -m $SRCIP_MASK -d $DSTIP -M $DSTIP_MASK -c $PROTOCOL -O $CONDITION -P $PORT -w $DIRECTION -D "${COMMENT}" -l $LOG
		elif [[ ${line} == deny* ]]; then
			genfilt -v $IPV -a D -s $SRCIP -m $SRCIP_MASK -d $DSTIP -M $DSTIP_MASK -c $PROTOCOL -O $CONDITION -P $PORT -w $DIRECTION -D "${COMMENT}" -l $LOG

		fi
	done < $FW_POLICY



	$MKFILT ‑g start
	$MKFILT -v4 -u
	$MKFILT -v6 -u


}

function rhel_firewall() {
	# firewall-cmd --state
		# firewall-cmd --get-default-zone
		# firewall-cmd --list-services
		# systemctl stop firewalld would clear it all


		# get the zones, 
			#firewall-cmd --get-zones

		# make a new one
			# firewall-cmd --new-zone=test1 --permanent
		
		# delete zone
			# firewall-cmd --delete-zone=<name>

	/usr/bin/firewall-cmd --state >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		systemctl stop firewalld
		systemctl disable firewalld
	fi

	# Iptables
	PROC_IPTABLES_NAMES=/proc/net/ip_tables_names
	NF_TABLES=$(cat "$PROC_IPTABLES_NAMES" 2>/dev/null)
	for i in $NF_TABLES; do
		iptables -t $i -F
		iptables -t $i -X
		iptables -t $i -Z
	done

	
	# Base policy
	while read line ; do

                IPV=$(echo $line | awk -F, '{print $2}')
                SRCIP=$(echo $line | awk -F, '{print $3}')
                SRCIP_MASK=$(echo $line | awk -F, '{print $4}')
                DSTIP=$(echo $line | awk -F, '{print $5}')
                DSTIP_MASK=$(echo $line | awk -F, '{print $6}')
                PROTOCOL=$(echo $line | awk -F, '{print $7}')
                CONDITION=$(echo $line | awk -F, '{print $8}')
                PORT=$(echo $line | awk -F, '{print $9}')
                DIRECTION=$(echo $line | awk -F, '{print $10}')
                COMMENT=$(echo $line | awk -F, '{print $11}')
                LOG=$(echo $line | awk -F, '{print $12}')

		if [[ $DIRECTION == inbound ]]; then
			DIRECTION=INPUT
		elif [[ $DIRECTION == outbound ]]; then
			DIRECTION=OUTPUT
		fi

		if  [[ ${line} == disabled* ]]; then
			exit
		elif  [[ ${line} == permit* ]]; then
			if [[ $LOG == log ]] ; then
				iptables -A ${DIRECTION} -m conntrack --ctstate  NEW -m ${PROTOCOL} -p ${PROTOCOL} -s ${SRCIP}/${SRCIP_MASK} --dport ${PORT} -m limit --limit 10/minute --limit-burst 2 -j LOG  --log-level info --log-prefix "${COMMENT}" --log-tcp-sequence  --log-tcp-options  --log-ip-options 
			fi

			iptables -A ${DIRECTION} -m conntrack --ctstate  NEW -m ${PROTOCOL} -p ${PROTOCOL} -s ${SRCIP}/${SRCIP_MASK} --dport ${PORT} -j ACCEPT

			
		elif [[ ${line} == deny* ]]; then
			if [[ $LOG == log ]] ; then
				iptables -A ${DIRECTION} -m conntrack --ctstate  NEW -m ${PROTOCOL} -p ${PROTOCOL} -s ${SRCIP}/${SRCIP_MASK} --dport ${PORT}  -m limit --limit 10/minute --limit-burst 2 -j LOG  --log-level info --log-prefix "${COMMENT}"  --log-tcp-sequence  --log-tcp-options  --log-ip-options
			fi
			iptables -A ${DIRECTION} -m conntrack --ctstate  NEW -m ${PROTOCOL} -p ${PROTOCOL} -s ${SRCIP}/${SRCIP_MASK} --dport ${PORT} -j DROP
		fi

		
	done < $FW_POLICY

	#echo COMMIT >> $FW_CONF
	#iptables-restore --noflush $FW_CONF
	
}



# Main
dist_detection
if [[ "$PKG" == "rpm" ]]; then
	rhel_firewall

elif [[ "$PKG" == "aix" ]]; then
	aix_firewall	
fi

